export function getFileMeta(config, ctx, filePath) {
    var fileMeta = ctx.files.get(filePath);
    if (fileMeta) {
        return Promise.resolve(fileMeta);
    }
    return readFile(config.packages, filePath).then(function (srcText) {
        return createFileMeta(config.packages, ctx, filePath, srcText);
    });
}
export function createFileMeta(packages, ctx, filePath, srcText) {
    var fileMeta = {
        fileName: packages.path.basename(filePath),
        filePath: filePath,
        fileExt: packages.path.extname(filePath),
        srcDir: packages.path.dirname(filePath),
        srcText: srcText,
        jsFilePath: null,
        jsText: null,
        isTsSourceFile: isTsSourceFile(filePath),
        hasCmpClass: false,
        cmpMeta: null,
        cmpClassName: null
    };
    if (fileMeta.isTsSourceFile) {
        fileMeta.hasCmpClass = hasCmpClass(fileMeta.srcText);
    }
    ctx.files.set(filePath, fileMeta);
    return fileMeta;
}
export function readFile(packages, filePath) {
    return new Promise(function (resolve, reject) {
        packages.fs.readFile(filePath, 'utf-8', function (err, data) {
            if (err) {
                reject(err);
            }
            else {
                resolve(data);
            }
        });
    });
}
export function writeFile(packages, filePath, content) {
    return new Promise(function (resolve, reject) {
        packages.fs.writeFile(filePath, content, function (err) {
            if (err) {
                reject(err);
            }
            else {
                resolve();
            }
        });
    });
}
export function copyFile(packages, src, dest) {
    return readFile(packages, src).then(function (content) {
        return writeFile(packages, dest, content);
    });
}
export function writeFiles(packages, files) {
    var paths = [];
    files.forEach(function (content, filePath) {
        content;
        paths.push(filePath);
    });
    return ensureDirs(packages, paths).then(function () {
        var promises = [];
        files.forEach(function (content, filePath) {
            promises.push(writeFile(packages, filePath, content));
        });
        return Promise.all(promises);
    });
}
export function ensureDirs(packages, paths) {
    var path = packages.path;
    var fs = packages.fs;
    var checkDirs = [];
    paths.forEach(function (p) {
        var dir = path.dirname(p);
        if (checkDirs.indexOf(dir) === -1) {
            checkDirs.push(dir);
        }
    });
    checkDirs = checkDirs.sort(function (a, b) {
        if (a.split(path.sep).length < b.split(path.sep).length) {
            return -1;
        }
        if (a.split(path.sep).length > b.split(path.sep).length) {
            return 1;
        }
        if (a.length < b.length) {
            return -1;
        }
        if (a.length > b.length) {
            return 1;
        }
        if (a < b) {
            return -1;
        }
        if (a > b) {
            return 1;
        }
        return 0;
    });
    var dirExists = new Set();
    return new Promise(function (resolve, reject) {
        function checkDir(resolve) {
            var dir = checkDirs.shift();
            if (!dir) {
                resolve();
                return;
            }
            var chunks = dir.split(path.sep);
            checkChunk(chunks, 0, resolve);
        }
        function checkChunk(chunks, appendIndex, resolve) {
            if (appendIndex >= chunks.length - 1) {
                checkDir(resolve);
                return;
            }
            var dir = chunks.slice(0, appendIndex + 2).join(path.sep);
            if (dirExists.has(dir)) {
                checkChunk(chunks, ++appendIndex, resolve);
                return;
            }
            fs.access(dir, function (err) {
                if (err) {
                    // no access
                    fs.mkdir(dir, function (err) {
                        if (err) {
                            reject(err);
                        }
                        else {
                            checkChunk(chunks, ++appendIndex, resolve);
                        }
                    });
                }
                else {
                    // has access
                    dirExists.add(dir);
                    checkChunk(chunks, ++appendIndex, resolve);
                }
            });
        }
        checkDir(resolve);
    });
}
export function isTsSourceFile(filePath) {
    var parts = filePath.toLowerCase().split('.');
    if (parts.length > 1) {
        if (parts[parts.length - 1] === 'ts') {
            if (parts.length > 2 && parts[parts.length - 2] === 'd') {
                return false;
            }
            return true;
        }
    }
    return false;
}
export function hasCmpClass(sourceText) {
    return (sourceText.indexOf('@Component') > -1);
}
export function logError(results, msg) {
    results.errors = results.errors || [];
    results.errors.push(msg);
    return results;
}
